/*
 * ovmapi_providers.h
 *
 *  Copyright (C) 2010, 2010 Oracle and/or its affiliates. All rights reserved.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation, version 2.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 *  MA 021110-1307, USA
 *
 *  Author : Wiekus.Beukes@oracle.com
 *
 */

#ifndef OVMAPI_PROVIDERS_H_
#define OVMAPI_PROVIDERS_H_

#include "OVMAPI.h"

#define PROVIDER_FAILURE                -1

#define PROVIDER_MAX_PREFIX_LEN         33
#define PROVIDER_MAX_NAME_LEN           256

#ifdef __x86_64__
#define OVMAPI_PROVIDER_DIR             "/usr/libexec/ovmapi/x86_64"
#else
#define OVMAPI_PROVIDER_DIR             "/usr/libexec/ovmapi/i386"
#endif

typedef struct _Provider
{
   int                                  prio;
   char                                 name[PROVIDER_MAX_NAME_LEN + 1];
   char                                 prefix[PROVIDER_MAX_PREFIX_LEN + 1];
   void*                                handle;
   OVMAPI_EventType                     eventMask;
   OVMAPI_Boolean                       (*Constructor)(void);
   void                                 (*Destructor)(void);

   /*
    * These are all the public functions a provider can define.
    * Only the Register and UnRegister are required, rest are optional.
    */
   OVMAPI_Status                        (*OVMAPI_Register)(int, OVMAPI_EventHandler, void*, OVMAPI_Handle*);
   OVMAPI_Status                        (*OVMAPI_UnRegister)(OVMAPI_Handle);
   OVMAPI_Status                        (*OVMAPI_ParamGetValue)(OVMAPI_Handle, const char*, char*, unsigned long*, unsigned long);
   OVMAPI_Status                        (*OVMAPI_ParamSetValue)(OVMAPI_Handle, const char*, const char *, OVMAPI_Boolean);
   OVMAPI_Status                        (*OVMAPI_Subscribe)(OVMAPI_Handle, OVMAPI_EventType);
   OVMAPI_Status                        (*OVMAPI_UnSubscribe)(OVMAPI_Handle, OVMAPI_EventType);
   OVMAPI_Status                        (*OVMAPI_EventComplete)(OVMAPI_Handle, OVMAPI_Event*, OVMAPI_Status);
   OVMAPI_Status                        (*OVMAPI_ParamGetAllNames)(OVMAPI_Handle, const char*, unsigned long*);
   OVMAPI_Status                        (*OVMAPI_ParamGetCount)(OVMAPI_Handle, unsigned long*);
   OVMAPI_Status                        (*OVMAPI_ParamDelete)(OVMAPI_Handle, const char*);
   OVMAPI_Status                        (*OVMAPI_SystemEventPublish)(OVMAPI_Handle, const char*, OVMAPI_EventSeverity, const char*, const char*, const char*);
   OVMAPI_Status                        (*OVMAPI_UserEventPublish)(OVMAPI_Handle, unsigned long, OVMAPI_EventSeverity, const char*, const char*, const char*);
   unsigned long                        (*OVMAPI_ParamGetValueSize)(OVMAPI_Handle, const char*);
   int                                  (*OVMAPI_GetSessionFileDescriptor)(OVMAPI_Handle);
   OVMAPI_Status                        (*OVMAPI_GetEvent)(OVMAPI_Handle, OVMAPI_Event**);
   OVMAPI_Status                        (*OVMAPI_EventPublish)(OVMAPI_Handle, OVMAPI_Event*);
   OVMAPI_Status                        (*OVMAPI_EventPost)(OVMAPI_Handle, OVMAPI_Event*);

} Provider;

typedef struct _ProviderIterator
{
    int                                 hasMore;
    void*                               providerNode;
} ProviderIterator;

void                                    openProviders(void);
void                                    closeProviders(void);

int                                     initProviderIterator(ProviderIterator* iterator);
int                                     providerIteratorHasNext(ProviderIterator iterator);
Provider*                               getNextProvider(ProviderIterator* iterator);
void                                    foreachProvider(void (*doFunc)(Provider*, void*), void* userData);

#endif /* OVMAPI_PROVIDERS_H_ */
